import readline
readline.parse_and_bind("tab: menu-complete")
import getch



def rlinput(prompt, prefill=''):
    readline.set_startup_hook(lambda: readline.insert_text(prefill))
    try:
       return input(prompt)
    finally:
       readline.set_startup_hook()

class Completer:

    def __init__(self, words):

        self.words = words
        self.prefix = None

    def complete(self, prefix, index):
        if prefix != self.prefix:
            # we have a new prefix!
            # find all words that start with this prefix
            self.matching_words = [
                w for w in self.words if w.startswith(prefix)
                ]
            self.prefix = prefix
        try:
            return self.matching_words[index]
        except IndexError:
            return None


def query_choice(choices,
                 prompt="Choice: ",
                 default=None, single_char=False,
                 allow_other=False,
                 case_insensitive=False):


    while True:
        if single_char:
            print(prompt, end=' ')
            choice = getch.getche()
            print()
        else:
            completer = Completer(choices)
            readline.set_completer(completer.complete)
            choice = rlinput(prompt=prompt, prefill=default or None)
            readline.set_completer(None)

        if not allow_other:
            if (
                    (case_insensitive and choice not in choices)
                    or (not case_insensitive
                        and choice.lower()
                        not in [ c.lower() for c in choices ]
                    )
            ):
                print("%s not one of: %s" %(choice, ','.join(choices)))
                continue
        return choice

def query_yes_no(prompt=">>> ", default="y", single_char=False):

    """Ask a yes/no question via raw_input() and return their answer.

    "question" is a string that is presented to the user.
    "default" is the presumed answer if the user just hits <Enter>.
        It must be "yes" (the default), "no" or None (meaning
        an answer is required of the user).

    The "answer" return value is one of "yes" or "no".
    """
    valid = {"yes":True,   "y":True,  "ye":True,
             "no":False,     "n":False}
    if default == None:
        choices = " [y/n] "
    elif default in ["yes", "y"]:
        choices = " [Y/n] "
    elif default in ["no", "n"]:
        choices = " [y/N] "
    else:
        raise ValueError("invalid default answer: '%s'" % default)

    choice = query_choice("yn", prompt=prompt + choices,
                          default=default,
                          single_char=single_char, case_insensitive=True)
    if default is not None and choice == '':
        return valid[default]
    elif choice in valid:
        return valid[choice]

readline.parse_and_bind("tab: menu-complete")

# print query_yes_no(single_char=True)
# print query_choice(["foo", "bar", "baz"])
# print query_choice(["foo", "bar", "baz"], default="foo", allow_other=True)



import logging
import os
import sys

LOG_LEVEL_DEFAULT=3
LOG_LEVELS = [
    "critical",
    "error",
    "warning",
    "info",
    "debug",
    "trace"
]

DEFAULT_FORMAT="%(asctime)s [%(module)16s:%(lineno)-4d] [%(levelname)8s] %(message)s"

log_level = None
stdout_handler = None
logger = None
formatter = None

def set_stdout_level(level=log_level):
    global stdout_handler
    assert(stdout_handler)

    try:
        handler = next(h for h in logger.handlers if h.stream == sys.stdout)
    except StopIteration:
        return

    handler.setLevel(level)

def add_log_handler(handler):
    handler.setFormatter(formatter)
    logger.addHandler(handler)


def setup_logging(level=0, handlers=[], log_format=DEFAULT_FORMAT, quiet_stdout=False):

    global logger
    global log_level
    global stdout_handler
    global formatter

    log_level = LOG_LEVEL_DEFAULT + level
    if log_level < 0 or log_level >= len(LOG_LEVELS):
        raise Exception("bad log level: %d" %(log_level))
    # add "trace" log level
    TRACE_LEVEL_NUM = 9
    logging.addLevelName(TRACE_LEVEL_NUM, "TRACE")
    logging.TRACE = TRACE_LEVEL_NUM
    def trace(self, message, *args, **kws):
        if self.isEnabledFor(TRACE_LEVEL_NUM):
            self._log(TRACE_LEVEL_NUM, message, args, **kws)
    logging.Logger.trace = trace

    if isinstance(level, str):
        log_level = getattr(logging, log_level.upper())
    else:
        log_level = getattr(logging, LOG_LEVELS[log_level].upper())

    if not isinstance(handlers, list):
        handlers = [handlers]

    logger = logging.getLogger()
    formatter = logging.Formatter(
        log_format,
        datefmt="%Y-%m-%d %H:%M:%S"
    )
    logger.setLevel(log_level)
    stdout_handler = logging.StreamHandler(sys.stdout)
    set_stdout_level(logging.CRITICAL if quiet_stdout else log_level)

    handlers.insert(0, stdout_handler)
    # if not handlers:
    #     handlers = [logging.StreamHandler(sys.stdout)]
    for handler in handlers:
        add_log_handler(handler)

    # logger = logging.basicConfig(
    #     level=level,
    #     format="%(asctime)s [%(module)16s:%(lineno)-4d] [%(levelname)8s] %(message)s",
    #     datefmt="%Y-%m-%d %H:%M:%S"
    # )

    # FIXME: move this elsewhere
    logging.getLogger("requests").setLevel(log_level+1)
    logging.getLogger("urllib3").setLevel(log_level+1)
    return logger

__all__ = [
    "setup_logging",
    "set_stdout_level",
    "add_log_handler"
]
